package com.acquisition.config;


import com.acquisition.mapper.auto.DataAcquisitionMapper;
import com.acquisition.model.auto.DataAcquisition;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.File;
import java.io.FileInputStream;
import java.util.ArrayList;
import java.util.List;
@Component
public class ExcelImporter {

    @Autowired
    private DataAcquisitionMapper dataAcquisitionMapper;

    /**
     * 配置总表数据导入读取Excel文件
     * @param folderPath
     * @throws Exception
     */
    public void importExcelToDatabase(String folderPath) throws Exception {
        // 文件夹路径
        File folder = new File(folderPath);
        if (!folder.exists()) {
            System.out.println("文件夹不存在：" + folderPath);
            return;
        }

        // 清空表数据
        clearTable();

        // 遍历文件夹中的 Excel 文件
        for (File file : folder.listFiles()) {
            if (file.isFile() && file.getName().endsWith(".xlsx")) {
                System.out.println("处理文件：" + file.getName());

                // 读取 Excel 文件
                FileInputStream fis = new FileInputStream(file);
                Workbook workbook = new XSSFWorkbook(fis);
                Sheet sheet = workbook.getSheetAt(0);

                // 解析 Excel 数据
                List<DataAcquisition> dataList = new ArrayList<>();
                for (int i = 1; i <= sheet.getLastRowNum(); i++) {
                    Row row = sheet.getRow(i);
                    if (row == null) continue;

                    DataAcquisition data = new DataAcquisition();
                    data.setIec104Addr(parseInteger(getCellValue(row.getCell(0))));
                    data.setIec104Xmax(parseInteger(getCellValue(row.getCell(1))));
                    data.setIec104Xmin(parseInteger(getCellValue(row.getCell(2))));
                    data.setIec104Coeff(parseInteger(getCellValue(row.getCell(3))));
                    data.setIec104Base(parseInteger(getCellValue(row.getCell(4))));
                    data.setGyfpAddr(parseInteger(getCellValue(row.getCell(5))));
                    data.setGyfpValid(parseInteger(getCellValue(row.getCell(6))));
                    data.setGyfpCoeff(parseInteger(getCellValue(row.getCell(7))));
                    data.setGyfpBase(parseInteger(getCellValue(row.getCell(8))));
                    data.setGyfp2Addr(parseInteger(getCellValue(row.getCell(9))));
                    data.setGyfp2Valid(parseInteger(getCellValue(row.getCell(10))));
                    data.setGyfp2Coeff(parseInteger(getCellValue(row.getCell(11))));
                    data.setGyfp2Base(parseInteger(getCellValue(row.getCell(12))));
                    data.setMongoKey(getCellValue(row.getCell(13))); // 字符串类型
                    data.setDescribe(getCellValue(row.getCell(14))); // 字符串类型
                    dataList.add(data);
                }

                // 将数据批量插入到数据库
                if (!dataList.isEmpty()) {
                    dataList.forEach(dataAcquisitionMapper::insert);
                }

                // 关闭 Excel 工作簿
                workbook.close();
                fis.close();
                System.out.println("文件处理完成：" + file.getName());
            }
        }
    }



    /**
     * 清空表数据
     */
    private void clearTable() {
        System.out.println("清空表数据...");
        dataAcquisitionMapper.delete(null); // 使用 MyBatis-Plus 提供的 delete 方法清空表
        System.out.println("表数据已清空！");
    }

    private String getCellValue(Cell cell) {
        if (cell == null) {
            return null;
        }
        switch (cell.getCellType()) {
            case STRING:
                return cell.getStringCellValue();
            case NUMERIC:
                // 检查是否为整数
                if (cell.getNumericCellValue() % 1 == 0) {
                    return String.valueOf((int) cell.getNumericCellValue());
                } else {
                    return String.valueOf(cell.getNumericCellValue());
                }
            case BOOLEAN:
                return String.valueOf(cell.getBooleanCellValue());
            case FORMULA:
                return cell.getCellFormula();
            default:
                return null;
        }
    }

    private Integer parseInteger(String value) {
        try {
            return Integer.parseInt(value);
        } catch (NumberFormatException e) {
            return null; // 如果无法解析，返回 null
        }
    }


}
