package com.acquisition.controller;

import com.acquisition.service.DataAcquisition.DataAcquisitionService;
import com.acquisition.service.DataAcquisition.FileProcessorConfigService;
import com.acquisition.service.DataAcquisition.FileProcessorService;
import com.acquisition.service.auto.impl.DataAcquisitionServiceImpl;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;

import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author wang
 * @since 2025-01-07
 */
@RestController
@RequestMapping("//data-acquisition")
public class DataAcquisitionController {

    @Autowired
    private DataAcquisitionServiceImpl dataAcquisitionService;

    @Autowired
    private DataAcquisitionService dataService;

    @Autowired
    private FileProcessorService fileProcessorService;

    @Autowired
    private FileProcessorConfigService configService;

    /**
     * 总表数据自动导入。
     *
     * @return 导入完成信息
     */
    @GetMapping("/import")
    @Operation(summary = "总表数据自动导入", description = "执行总表数据的自动导入任务")
    @ApiResponse(responseCode = "200", description = "成功返回导入完成信息",
            content = @Content(mediaType = "text/plain",
                    schema = @Schema(implementation = String.class)))
    public String importData() {
        dataAcquisitionService.importDataFromExcel();
        return "数据导入完成！";
    }


    /**
     * 可选列导出SQLite。
     *
     * @param columns 需要导出的字段列表
     * @return 导出完成信息
     * @throws Exception 如果导出过程中出现问题
     */
    @GetMapping("/export-to-sqlite")
    @Operation(summary = "可选列导出SQLite", description = "根据指定的字段导出数据到SQLite数据库")
    @ApiResponse(responseCode = "200", description = "成功返回导出完成信息",
            content = @Content(mediaType = "text/plain",
                    schema = @Schema(implementation = String.class)))
    public String exportToSQLite(
            @Parameter(description = "需要导出的字段名称列表", required = true)
            @RequestParam List<String> columns) throws Exception {
        dataService.export(columns);
        return "数据已导出";
    }


    /**
     * 创建data文件并复制demo文件。
     *
     * @return 完成信息
     * @throws Exception 如果处理文件时出现错误
     */
    @GetMapping("/fileprocess")
    @Operation(summary = "创建data文件并复制demo文件", description = "执行创建data文件并复制demo文件")
    @ApiResponse(responseCode = "200", description = "成功返回完成信息",
            content = @Content(mediaType = "text/plain",
                    schema = @Schema(implementation = String.class)))
    public String fileprocess() throws Exception {
        fileProcessorService.processFiles();
        return "文件夹创建已完成";
    }

    /**
     * 数据自动采集。
     *
     * @return 完成信息
     * @throws Exception 如果处理文件时出现错误
     */
    @GetMapping("/fileprocessconfig")
    @Operation(summary = "数据自动采集配置", description = "数据自动采集配置")
    @ApiResponse(responseCode = "200", description = "成功返回完成信息",
            content = @Content(mediaType = "text/plain",
                    schema = @Schema(implementation = String.class)))
    public String fileprocessconfig() throws Exception {
        configService.processFiles();
        return "数据自动配置已完成";
    }

}
