package com.acquisition.service.DataAcquisition;

import com.acquisition.mapper.auto.DataAcquisitionMapper;
import com.acquisition.model.auto.DataAcquisition;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.File;
import java.lang.reflect.Field;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.util.Collections;
import java.util.List;

@Service
public class DataAcquisitionService {
    @Autowired
    private DataAcquisitionMapper dataAcquisitionMapper;

    /**
     * 动态选择列导出sqlite表
     *
     * @param columns
     * @return
     * @throws Exception
     */
    public String export(List<String> columns) throws Exception {
        // 将列名组成的字符串传递给 SQL 查询
        String columnList = String.join(",", columns);

        // 动态构建 SQL 查询
        List<DataAcquisition> dataList = dataAcquisitionMapper.selectList(new QueryWrapper<DataAcquisition>()
                .select(columns.toArray(new String[0]))); // 动态选择列

        // 设置文件路径，目标路径为桌面上的 data_acquisition/sqlite 文件夹
        String sqliteFilePath = System.getProperty("user.home") + "\\Desktop\\data_acquisition\\sqlite\\keytable.sqlite3";

        // 确保目标文件夹存在
        File folder = new File(System.getProperty("user.home") + "\\Desktop\\data_acquisition\\sqlite");
        if (!folder.exists()) {
            folder.mkdirs();  // 如果文件夹不存在，则创建它
        }

        try (Connection conn = DriverManager.getConnection("jdbc:sqlite:" + sqliteFilePath)) {
            // 创建表的 SQL 语句，根据用户输入的列生成表结构
            StringBuilder createTableSQL = new StringBuilder("CREATE TABLE IF NOT EXISTS key_table (");

            // 获取 DataAcquisition 类中的字段类型来构造 SQL 表
            for (String column : columns) {
                // 将列名从下划线格式转换为驼峰命名法
                String camelCaseColumn = convertToCamelCase(column);
                Field field = DataAcquisition.class.getDeclaredField(camelCaseColumn);  // 获取对应字段
                String fieldType = getSQLType(field.getType()); // 获取 SQL 数据类型
                createTableSQL.append(column).append(" ").append(fieldType).append(", ");
            }
            createTableSQL.setLength(createTableSQL.length() - 2);  // 删除最后一个多余的逗号
            createTableSQL.append(")");

            conn.createStatement().execute(createTableSQL.toString());

            // 动态生成插入语句
            String insertSQL = "INSERT INTO key_table (" + columnList + ") VALUES (" +
                    String.join(",", Collections.nCopies(columns.size(), "?")) + ")";
            try (PreparedStatement pstmt = conn.prepareStatement(insertSQL)) {
                for (DataAcquisition data : dataList) {
                    for (int i = 0; i < columns.size(); i++) {
                        // 根据列名动态构造 getter 方法名（转换下划线为驼峰格式）
                        String getterMethodName = "get" + capitalizeFirstLetter(convertToCamelCase(columns.get(i)));
                        Object value = data.getClass().getMethod(getterMethodName).invoke(data);
                        pstmt.setObject(i + 1, value);
                    }
                    pstmt.addBatch(); // 批量插入
                }
                pstmt.executeBatch(); // 执行批量插入
            }
            return "数据已成功导出到 SQLite 文件：" + sqliteFilePath;
        } catch (Exception e) {
            e.printStackTrace();
            return "导出失败: " + e.getMessage();
        }
    }


    // 工具方法：将下划线格式转换为驼峰命名法
    private String convertToCamelCase(String str) {
        String[] parts = str.split("_");
        StringBuilder camelCaseString = new StringBuilder(parts[0].toLowerCase());
        for (int i = 1; i < parts.length; i++) {
            camelCaseString.append(parts[i].substring(0, 1).toUpperCase())
                    .append(parts[i].substring(1).toLowerCase());
        }
        return camelCaseString.toString();
    }

    // 工具方法：处理字段名，确保_后第一字母大写
    private String capitalizeFirstLetter(String str) {
        if (str == null || str.isEmpty()) {
            return str;
        }
        return str.substring(0, 1).toUpperCase() + str.substring(1);
    }

    // 获取字段的 SQL 数据类型
    private String getSQLType(Class<?> clazz) {
        if (clazz == Integer.class || clazz == int.class) {
            return "INTEGER";
        } else if (clazz == String.class) {
            return "TEXT";
        } else if (clazz == Double.class || clazz == double.class) {
            return "REAL";
        } else if (clazz == Boolean.class || clazz == boolean.class) {
            return "INTEGER";
        } else if (clazz == java.util.Date.class) {
            return "TEXT";
        }
        return "TEXT";
    }
}
